#ifndef __PKCRYPT_H
#define __PKCRYPT_H

#include <IlsPlat.h>

#ifdef __cplusplus
extern "C" {
#endif


#define SYSFPRINTBUFSIZE	16384 /* system fingerprint buffer size */


/**
 * Create a private / public key pair based on a given seed value.
 * @param pwd: password needed to use this API
 * @param keySeed: seed for key creation.
 * @param strPubkey: buffer to receive public key
 * @param pubkeyBufsize: public key buffer size
 * @param strPvtkey: buffer to receive private key
 * @param pvtkeyBufsize: private key buffer size
 * @return status code: 0 if success, -1 if error
 **/
DLLEXPORT int CALLING_CONVENTION makeKeyPair(const char *pwd, const char *keySeed,
									char strPubkey[], size_t pubkeyBufsize,
									char strPvtkey[], size_t pvtkeyBufsize);

/**
 * Create a public key based on a given seed value.
 * @param keySeed: seed for key creation.
 * @param strPubkey: buffer to receive public key
 * @param pubkeyBufsize: public key buffer size
 * @return status code: 0 if success, -1 if error
 **/
DLLEXPORT int CALLING_CONVENTION makePublicKey(const char *keySeed,
									char strPubkey[], size_t pubkeyBufsize);


/**
 * encryptWithPublicKey:
 * Encrypt specified text with a specified public key.
 * @param cleartext: the text to encrypt
 * @param publicKey: the public key to use for encryption
 * @param strEncodedData: pointer to buffer to receive encrypted data
 * @param bufferSize: size of buffer receiving encrypted data
 * @return status code: 0 if success, -1 if error such as
 * buffer overflow
 **/
DLLEXPORT int CALLING_CONVENTION encryptWithPublicKey(
							const char *cleartext, 
							const char *publicKey,
							char strEncodedData[],
							size_t bufferSize);

/**
 * decryptWithPublicKey:
 * Decrypt specified encrypted text with a specified public key.
 * @param crypttext: the encrypted text to decrypt
 * @param publicKey: the public key to use for decryption
 * @param strDecodedData: pointer to buffer to receive decrypted data
 * @param bufferSize: size of buffer receiving decrypted data
 * @return status code: 0 if success, -1 if error such as
 * buffer overflow
 **/
DLLEXPORT int CALLING_CONVENTION decryptWithPublicKey(
							const char *crypttext, 
							const char *publicKey,
							char strDecodedData[],
							size_t bufferSize);

/**
 * encryptWithPrivateKey:
 * Encrypt specified text with a specified private key.
 * @param password: password for using this API
 * @param cleartext: the text to encrypt
 * @param privateKey: the private key to use for encryption
 * @param strEncodedData: pointer to buffer to receive encrypted data
 * @param bufferSize: size of buffer receiving encrypted data
 * @return status code: 0 if success, -1 if error such as
 * buffer overflow
 **/
DLLEXPORT int CALLING_CONVENTION encryptWithPrivateKey(
							const char *password,
							const char *cleartext, 
							const char *privateKey,
							char strEncodedData[],
							size_t bufferSize);

/**
 * decryptWithPrivateKey:
 * Decrypt specified encrypted text with a specified private key.
 * @param password: password for using this API
 * @param crypttext: the encrypted text to decrypt
 * @param privateKey: the privatekey to use for decryption
 * @param strDecodedData: pointer to buffer to receive decrypted data
 * @param bufferSize: size of buffer receiving decrypted data
 * @return status code: 0 if success, -1 if error such as
 * buffer overflow
 **/
DLLEXPORT int CALLING_CONVENTION decryptWithPrivateKey(
							const char *password,
							const char *crypttext, 
							const char *publicKey,
							char strDecodedData[],
							size_t bufferSize);

/**
 * Make a tamper-proof encrypted return receipt for secure
 * self-service sneakernet transmission.  The output is compatible 
 * with the Orion sneakernet web self-service facility.
 * @param licenseKey: the license key being returned
 * @param productName: name of product associated with the license key
 * @param password: secret password that was used to encrypt the
 *     license key
 * @param userParams: any user-defined parameters that are to be
 *     saved in the server.
 * @param strReceiptBuf: buffer to receive return receipt
 * @param receiptBufsize: size of return receipt buffer
 * @return status code: 0 if success, -1 if error such as buffer overflow
 **/
DLLEXPORT int CALLING_CONVENTION makeReturnReceipt(
						const char *licenseKey, 
						const char *productName,
						const char *password, 
						const char *userParams,
						char  strReceiptBuf[],
						size_t receiptBufsize
						);


/**
 * Make a tamper-proof encrypted system fingerprint for secure
 * self-service sneakernet transmission.  The output is compatible 
 * with the Orion sneakernet web self-service facility.
 * @param rawLicenseKey: the raw license key loaded with license
 *		parameters to use as a basis for the fingerprint.  The 
 *      license parameters include the machine fingerprint and
 *      identification information for the destination Orion service.
 * @param productName: name of product associated with the license key
 * @param password: secret password that was used to encrypt the
 *     raw license key
 * @param checkoutDuration: desired activation duration in seconds
 * @param options: predicate for any options that are to be specified for
 *     a feature checkout.
 * @param strSysfprintBuf: buffer to receive system fingerprint
 * @param sysfprintBufsize: size of system fingerprint buffer
 * @return status code: 0 if success, -1 if error such as buffer overflow
 **/
DLLEXPORT int CALLING_CONVENTION makeSystemFingerprint(
						const char *rawLicenseKey, 
						const char *productName,
						const char *password, 
						long 		checkoutDuration,
						const char *options,
						char		strSysfprintBuf[],
						size_t		sysfprintBufsize
						);


/**
 * Encrypt Orion-specific cleartext with a specified secret key.
 * Can't be used for general-purpose encryption / decryption.
 * @param cleartext: clear text to encrypt
 * @param password: secret key to use for encryption
 * @param strEncodedData: buffer to receive encoded data
 * @param encodedDataBufsize: size of encoded-data buffer
 * @return status: 0 if success, -1 if error
 **/
DLLEXPORT int CALLING_CONVENTION encryptIls(
						const char *cleartext, 
						const char *password,
						char strEncodedData[],
						size_t encodedDataBufsize);

/**
 * Decrypt Orion-specific cyphertext with a specified secret key.
 * Can't be used for general-purpose encryption / decryption.
 * @param cyphertext: cyphertext to decrypt
 * @param password: secret key to use for decryption
 * @param strDecodedData: buffer to receive decoded data
 * @param decodedDataBufsize: size of decoded-data buffer
 * @return status: 0 if success, -1 if error
 **/
DLLEXPORT int CALLING_CONVENTION decryptIls(
						const char *cyphertext, 
						const char *password,
						char strDecodedData[],
						size_t decodedDataBufsize);

#ifdef __cplusplus
}
#endif


#endif

